type t = Mustache.t

module Configuration = struct
  type path_t = Fpath.t option

  type paths_t = {
      dir    : path_t;
      header : path_t;
      note   : path_t;
      front  : path_t;
      list   : path_t;
      item   : path_t;
    }

  let of_config config =
    let open Confix.ConfixToml in
    let p k = path config ("templates" / k) in
    {
      dir    = p "dir";
      header = p "header";
      note   = p "note";
      front  = p "front";
      list   = p "list";
      item   = p "item";
    }
end

type header_t = Header of t
type footer_t = Footer of t
type list_t   = List of t
type item_t   = Item of t
type note_t   = Note of t
type front_t  = Front of t

let of_string = Mustache.of_string
let of_file f = File.load f |> of_string

let map_tpl_opt f field ps =
  let open Configuration in
  match ps.dir, field with
  | Some dir, Some bn -> Some (f (of_file (Fpath.append dir bn)))
  | _ -> None

let header ps = map_tpl_opt (fun v -> Header v) ps.Configuration.header ps
let note   ps = map_tpl_opt (fun v -> Note v)   ps.Configuration.note   ps
let front  ps = map_tpl_opt (fun v -> Front v)  ps.Configuration.front  ps
let list   ps = map_tpl_opt (fun v -> List v)   ps.Configuration.list   ps
let item   ps = map_tpl_opt (fun v -> Item v)   ps.Configuration.item   ps

let string s = s
let section ~inverted name contents = "section"
let unescaped u = List.fold_left (fun a s -> a ^ s) "" u
let partial ?indent name _ _ = "partials not supported"
let comment c = c
let concat l = String.concat "" l

let escaped_meta (meta : Logarion.Meta.t) e =
  let open Logarion in
  let e = List.hd e in
  match e with
  | "url" -> "/note/" ^ Meta.alias meta
  | "date" | "date_created" | "date_edited" | "date_published" | "date_human" ->
     "<time>" ^ Meta.value_with_name meta e ^ "</time>"
  | tag -> Meta.value_with_name meta tag
             
let escaped_note note e = match List.hd e with
  | "body" -> Omd.to_html @@ Omd.of_string note.Logarion.Note.body
  | _    -> escaped_meta note.Logarion.Note.meta e

let escaped_header blog_url title e = match List.hd e with
  | "blog_url" -> blog_url
  | "title"    -> title
  | tag -> prerr_endline ("unknown tag: " ^ tag); ""

let anchor_of_meta meta =
  let module Meta = Logarion.Meta in
  let open Html in
  anchor
    (Meta.alias meta ^ ".html")
    [ div ~style_class:"title" [data meta.Meta.title];
      div ~style_class:"time"  [data Meta.Date.(pretty_date (last meta.Meta.date))] ]

let listing metas =
  let open Html in
  list_unordered @@ List.map (fun m -> list_item @@ anchor_of_meta @@ m) metas

let fold_index ~from ~n metas =
  let escaped e =
    match List.hd e with
    | "navigation" ->
       ""
       ^ (if from > 0 then ("<a href=\"?p=" ^ string_of_int (pred from) ^ "\">previous</a> | ") else "")
       ^ (if n <= List.length metas then ("<a href=\"?p=" ^  string_of_int (succ from) ^ "\">next</a>") else "")
    | "recent_texts_listing" -> (* listing metas *) ""
    | "topics" ->
       let topics =
         ListLabels.fold_left
           ~init:(Logarion.Meta.StringSet.empty)
           ~f:(fun a e -> Logarion.Meta.unique_topics a e ) metas
       in
       Logarion.Meta.StringSet.fold (fun e a -> a ^ "<li><a href=\"/topic/" ^ e ^ "\">" ^ e ^ "</a></li>") topics ""
    | e -> prerr_endline ("unknown tag: " ^ e); "" in
  Mustache.fold ~string ~section ~escaped ~unescaped ~partial ~comment ~concat

let note_page ?(header_template=None) ?(note_template=None) ~style url title note =

  let header_html = match header_template with
    | Some (Header s) ->
       let data = Mustache.fold ~string ~section ~escaped:(escaped_header url title) ~unescaped ~partial ~comment ~concat s in
       Html.header @@ Html.data data
    | None -> Html.(header (heading1 title))
  in

  let note_html = match note_template with
    | Some (Note s) ->
       let data = Mustache.fold ~string ~section ~escaped:(escaped_note note) ~unescaped ~partial ~comment ~concat s in
       Html.data data
    | None ->
       let open Logarion.Note in
       let open Logarion.Meta in
       let date = Date.(pretty_date @@ last note.meta.date) in
       let abstract = note.meta.abstract in
       let body = Omd.to_html (Omd.of_string note.body) in
       Html.note ~date ~abstract ~body
  in

  Html.to_string @@ Html.page ~style url title header_html note_html

let listing_page ?(header_template=None) ?(listing_template=None) ~style ~from ~n url title metas =

  let header_html = match header_template with
    | Some (Header s) ->
       let data = Mustache.fold ~string ~section ~escaped:(escaped_header url title) ~unescaped ~partial ~comment ~concat s in
       Html.header @@ Html.data data
    | None -> Html.(header (heading1 title))
  in

  let listing_html = match listing_template with
    | Some (Note s) -> Html.data @@ fold_index ~from ~n metas s
    | None -> listing metas
  in

  Html.to_string @@ Html.page ~style url title header_html listing_html
